.\" 
Document: $Id: pccom8.ms,v 1.4 1996/11/07 16:01:11 mc Exp $
.TL
The Decision PCCOM8 Serial Card and Device Driver
.AU
Michael Cardell
.AI
Signum Support AB
Teknikringen 8
583 30 LINKPING
SWEDEN
.AB
This text describes the PCCOM8 multi-port serial card and the changes
made to the Linux serial device driver to support it. The text goes
through general configuration of the serial card as well as pointing
out flaws and possible fixes in the
.CW serial.c
device driver.
.AE
.NH
Introduction
.PP
Signum Support, a company specialising in free software support and
Linux, was approached by MYDATA Automation AB, a Swedish robotics
company, to write a Linux device driver for the Decision PCCOM8
multi-port serial card. The driver and this document was written by
Christer Weinigel and Michael Cardell.
.PP
We developed the driver using two identical RJ-11 type cards with a
basic 16450 UART, with two Kensington trackballs and a Qtronix
trackball keyboard. During tests, we ran 115.2 kb/s zmodem sessions on
several ports and used the trackball at the same time. We experienced
loss of packets on the zmodem sessions, but the trackball (at a mere
1200 b/s) didn't appear to lose any characters. We have only tested
the driver with the 16450 version of the board, but we think it should
work with 16550A and 16650 as well.
.PP
.NH
The Card
.PP
PCCOM8 is a serial adapter that provides eight asynchronous serial
communications ports (RS232C or RS422). It is manufactured by Decision
Computer Company, Ltd, Taipei, Taiwan:

.KS
  4/F No. 31
  Alley 4, Lane 36, Sec. 5 Meing-Shen E. Rd.
  Taipei
  Taiwan R.O.C.
.KE

The URL for their WWW pages is

  http://www.decision.com.tw

These addresses are listed here for informational purposes only. We
have nothing to do with Decision Computer Company, but we do hope that
they will point to this Linux device driver if people ask them if they
can use their serial card under Linux. 
.PP
It says in the 
.I
PCCOM8 User's Manual
.R
that the card is ARNET 8 multi-port card compatible. I guess that
means that the device driver might work with the ARNET 8 serial card
(whatever that is) as well. I don't know. I haven't got any ARNET 8
cards and I've never heard about them. If someone tries this driver
with an ARNET 8, please let us know if it worked. Better yet, if it
doesn't work, send us patches and we'll see if the driver still works
with a PCCOM8 card with the patches applied.
.PP
There are three switch banks on the card for setting IRQ, I/O Address
and Interrupt Vector respectively. The physical locations of these
banks may differ with different models of the PCCOM8 card, but on our
model, holding the card (either the RJ-11 version or the DB37 Pin
Board) with the bus contact down, they are located from left to right
on the top of the card in this order: I/O Address switch, IRQ switch
and Interrupt Vector switch.  The switches are zero (0) when in the ON
position and one (1) in the other, except in the case of the IRQ
switches, which are explained below.
.NH 2
I/O Address
.PP
The I/O address must be an even multiple of 0x40 since a block of
hexadecimal 40 (that is, 64 decimal) contiguous I/O locations must be
reserved from the start of the selected adress. The switch bank
contains 4 switches. Switch number 1 from the left is the most
significant. You get the address by multiplying the binary number you
get from the switches by hexadecimal 40. For instance, the settings

.KS
  0101 (that is: on, off, on, off)
  1234
.KE

would give the value of 0x5 which, in turn, would give the address
0x140 (5 * 40).
.NH 2
IRQ
.PP
The interrupt request line (IRQ) is set in a straightforward way,
where each switch is an IRQ number, counting from 2 to 7. If switch 1
is set to on, the IRQ is 2, if it is set to 2, the IRQ is 3, and so
on. 
.NH 2
Interrupt Vector (Interrupt Latch Register = IV + 2)
.PP
The interrupt vector address is set by the third switch bank's first
six switches. Switch number seven is the on/off switch and number
eight seems to be reserved. Number seven is to be set to ON at all
times, if you don't want to turn the interrupt vector off. If you do,
turn all switches to the off position.
.PP
The interrupt vector has to be an even multiple of 0x10. Divide the
address you want with 0x10 and set the switches to represent that
number in binary, with switches in the OFF position counting as 1s and
in the ON position as 0s. This means that:

.KS
  00001101
  12345678
.KE

with the switches 7 and 8 ignored, is (0x3) which gives the interrupt
vector at 0x30 (0x3 * 0x10).

.NH
Cables
.PP
The expansion cables, as delivered with the PCCOM8 cards, are a bit
mysterious. We didn't get things to work right out of the box with
things, such as the Kensington trackball, that needs RTS high. Our
solution to the problems with the cables was to construct a cable that
had the following schematics: A straight RS232 cable with RTS
following DTR.
.PP
If you want to make an RJ-11 to DB9 connector, it might look like
this:
.KS
			DSUB-9

case	5	3	2	8	6	4	7
.br
SGN	SGND	Tx	Rx	CTS	DSR	DTR	RTS
.br
 |	|	|	|	|-------|	|-------|
 |	|	|	|	|		|
.br
 1	2	3	4	5		6

			RJ-11 (Modular 6/6)
.KE
.PP
Please observe that DTR and RTS are both inputs and connecting them
like this will cause a short circuit if driven by equipment which
keeps CTS low and DSR high or vice versa (the RJ11 to DB25 cables
that come with the PCCOM8 card are wired this way too).
.NH
Configuration
.PP
To autoconfigure the PCCOM8 card and automatically check what IRQ it
emits, use these lines for each serial line that you will use (cua32
to cua39). This assumes that the definition of PCCOM8_BASE in the
source is correct. If you're uncertain, check and, if neccesary,
change and recompile the sources.

.KS
	setserial /dev/cua32 auto_irq irq 0
.br
	setserial /dev/cua32 autoconf
.KE

where /dev/cua32 should be changed to the real device you are
interested in autoconfiguring. The first setserial command enables the
auto_irq option and sets the initial IRQ to 0. At the second setserial
command, the driver is configured.
.NH
The Device Driver
.PP
We have used the hub6 variable in serial_struct to store the interrupt
vector for the PCCOM8 card. Hence, we don't allow for both the HUB6
card and the PCCOM8 card to be enabled at the same time; a truly
bletcherous hack, but it won't break binary compatibility for the
setserial structures. This really should be changed in Linux 2.1.  It
would be nice if the name of this variable could be changed to, say,
.CW driver_info,
and add a field which defines how driver_info will be interpreted,
say,
.CW serial_card. 
If 
.CW serial_card
is set to PCCOM8, 
.CW driver_info
would be the interrupt vector address, for instance.  Finally
setserial should be updated to be able to modify this variable.
.PP
For now, we 
.CW #define
CONFIG_PCCOM8 to know that we're going to use the PCCOM8
interpretation of hub6 and embed all PCCOM8 specific things in
\fC#ifdef\fRs.
.PP
In the async_struct rs_table, we use

.KS
  PCCOM8_BASE
  PCCOM8_IRQ
  PCCOM8_VECTOR
  PCCOM8_FLAGS
.KE

to set up the UART address (0x280), initial IRQ (0) and interrupt
vector (0x2c0) for eight ports. 
.PP
You can get the driver to autoconfigure the PCCOM8 card at boot by
adding ASYNC_BOOT_AUTOCONF to the flags, but don't do this unless
you're absolutely sure that the base address and interrupt vector
above are correct, your computer will most probably hang during boot
if they are incorrect. Since there is no other way of setting the
interrupt vector than to modify the source, this is a bit redundant
right now; if you know the interrupt vector I/O adresses and change
the source, it would probably be safe to autoconfigure the card too,
if you don't know the interrupt vector addresses, it won't work
anyway.
.PP
These defines are undefined right after rs_table and hence not used
elsewhere in the driver.
.PP
The 
.CW set_serial_info() 
function is called upon to initialize the ports if we use setserial,
and we initialize the PCCOM8 port by reading the interrupt latch
register with
.CW inb_p(). We initialize the PCCOM8 in 
.CW rs_init(), 
the serial driver boot-time initialization routine as well.
.PP
We also initialize in
.CW register_serial().
This function is used to configure serial ports at run-time by some
PCMCIA routines (it says in the sources).
.PP
For safety reasons, we clear the interrupt latch in
.CW get_auto_irq() 
as well, since an IRQ must have been triggered in order to figure out
the IRQ settings.
.PP
To handle an IRQ event, we added an inb_p from the interrupt latch
register (vector address + 2), in the generic interrupt routine
.CW rs_interrupt().  
At the interrupt latch address, there is a bitstring representing the
ports, with port eight being the most significant bit.
.PP
After having read the characters waiting for reading, we clear the
interrupt latch. However, we're not certain what happens if there are
several interrupt at the same time and we clear the entire register
from 
.I one
of them. It says in the source that rs_interrupt is a "fast" interrupt
and runs with interrupts turned off, so it seems to work fine.
.PP
Basically, we have done the same for 
.CW rs_interrupt_single().

.\" $Id: pccom8.ms,v 1.4 1996/11/07 16:01:11 mc Exp $
